#pragma once
#include <atl/object.hpp>
#include <ts/scenario/context.hpp>
#include <c_tracer/c_tracer.h>

#include <cassert>

namespace cpptesk
{
namespace ts
{

//
//      
//

struct scenario_method_base
{
    const char* name;
    scenario_method_base(const char * name) : name(name) {}

    virtual Object * init() = 0;
    virtual bool has_next(Object*) = 0;
    virtual bool next(Object* ) = 0;
    virtual bool invoke(Object* ) = 0;

    virtual ~scenario_method_base() {}
};

template <typename S, typename T>
struct scenario_method : virtual scenario_method_base
{
    typedef bool (S::*method_t)(typename cpptesk::atl::CppObject<T>::value_t& );
    S& ref;
    method_t method;
    scenario_method(const char * name, S& ref, method_t method) : scenario_method_base(name), ref(ref), method(method) {}

    virtual Object * init()
    {
        cpptesk::atl::CppObject<T> * result = cpptesk::atl::CppObject<T>::create();
        assert(result->ptr());
        result->ref().__mode = cpptesk::ts::INIT_ITERATION;
        if ((ref.*method)(result->ref()))
        {
            return result;
        }
        else
        {
            ::destroy_Object(result);
            return 0;
        }
    }

    virtual bool has_next(Object * arg)
    {
        if (!arg) return false;
        cpptesk::atl::CppObject<T> * obj = (cpptesk::atl::CppObject<T> *)arg;
        assert(obj->ptr());
        obj->ref().__mode = cpptesk::ts::NEXT_ITERATION;
        bool result = (ref.*method)(obj->ref());
        ::destroy_Object(arg);
        return result;
    }

    virtual bool next(Object * arg)
    {
        cpptesk::atl::CppObject<T> * obj = (cpptesk::atl::CppObject<T> *)arg;
        assert(obj->ptr());
        obj->ref().__mode = cpptesk::ts::NEXT_ITERATION;
        bool result = (ref.*method)(obj->ref());
        ::destroy_Object(arg);
        return result;
    }

    virtual bool invoke(Object * arg)
    {
        cpptesk::atl::CppObject<T> * obj = (cpptesk::atl::CppObject<T> *)arg;
        std::ostringstream out;
        out << *obj;
        ::traceScenarioValue(
            "iteration variable",
            TYPE(arg)->name,
            "iterator",
            out.str().c_str()
        );
        assert(obj->ptr());
        obj->ref().__mode = cpptesk::ts::INVOKE_ACTION;
        bool result = (ref.*method)(obj->ref());
        ::destroy_Object(arg);
        return result;
    }
};

//    
template <typename S, typename T>
struct scenario_method_no_context : virtual scenario_method_base
{
    typedef bool (S::*method_t)(void);
    S& ref;
    method_t method;
    scenario_method_no_context(const char * name, S& ref, method_t method) : scenario_method_base(name), ref(ref), method(method) {}

    virtual Object * init()
    {
        return cpptesk::atl::CppObject<int>::create();
    }

    virtual bool has_next(Object * arg)
    {
        cpptesk::atl::CppObject<int>* count = (cpptesk::atl::CppObject<int>*)arg;
        return (count->value++ == 0);
    }

    virtual bool next(Object * arg)
    {
        cpptesk::atl::CppObject<int>* count = (cpptesk::atl::CppObject<int>*)arg;
        return (count->value++ == 0);
    }

    virtual bool invoke(Object * arg)
    {
        std::ostringstream out;
        ::traceScenarioValue(
            "iteration variable",
            TYPE(arg)->name,
            "iterator",
            ""
        );
        bool result = (ref.*method)();
        return result;
    }
};

}
}
