#define CPPTESK_ITERATION_SHORT_SHORT_NAMES
#define CPPTESK_ITERATION__LOCAL_VARS_IN_ACTION
#include "account_scen.h"
#include <tracer/tracer.hpp>

using namespace cpptesk::ts;

bool AccountScenario::init(int argc, char ** argv)
{
    return true;
}

void AccountScenario::finish() {}

#ifdef CPPTESK_ITERATION__LOCAL_VARS_IN_ACTION
bool AccountScenario::deposit_scenario(IntCtx& ctx)
{
    IBEGIN
    for (IVAR(i) = 1; IVAR(i) + mediator.balance < limit; ++ IVAR(i))
    {
        if (IVAR(i)%2)
        {
            IACTION
            {
                int val = IVAR(i);
                int sum = mediator.deposit(val);
                YIELD (sum > 0 && sum <= val);
            }
        }
        else
        {
            IACTION
            {
                int val = IVAR(i);
                int sum = mediator.deposit(val);
                YIELD (sum > 0 && sum <= val);
            }
        }
    }
    IEND
}

bool AccountScenario::withdraw_scenario(IntCtx& ctx)
{
    IBEGIN
    for (IVAR(i) = 1; mediator.balance - IVAR(i) > -limit; ++ IVAR(i))
    {
        IACTION
        {
            int val = IVAR(i);
            int sum = mediator.withdraw(val);
            YIELD (sum >= 0 && sum <= val);
        }
    }
    IEND
}

bool AccountScenarioEx::random_withdraw_scenario(IntCtx& ctx)
{
    IBEGIN
    IACTION
    {
        int value = rand() % limit;
        mediator.withdraw(value);
        YIELD(true);
    }
    IEND
}

#else // No local variables
bool AccountScenario::deposit_scenario(IntCtx& ctx)
{
    IBEGIN
    for (IVAR(i) = 1; IVAR(i) + mediator.balance < limit; ++ IVAR(i))
    {
        IACTION
        {
            YIELD (mediator.deposit(IVAR(i)) > 0);
        }
    }
    IEND
}
bool AccountScenario::withdraw_scenario(IntCtx& ctx)
{
    IBEGIN
    for (IVAR(i) = 1; mediator.balance - IVAR(i) > -limit; ++ IVAR(i))
    {
        IACTION
        {
            mediator.withdraw(IVAR(i));
            YIELD (true);
        }
    }
    IEND
}

bool AccountScenarioEx::random_withdraw_scenario(IntCtx& ctx)
{
    IBEGIN
    IACTION
    {
        mediator.withdraw(rand() % limit);
        YIELD(true);
    }
    IEND
}

#endif
AccountScenario::AccountScenario(int limit, Account* impl) : ScenarioBase(), mediator(impl), limit(limit)
{
    setup("Account Test");
    CPPTESK_ADD_SCENARIO_METHOD(AccountScenario::deposit_scenario);
    CPPTESK_ADD_SCENARIO_METHOD(AccountScenario::withdraw_scenario);
}
AccountScenario::AccountScenario(void) : ScenarioBase()
{
    setup("Account Test");
    CPPTESK_ADD_SCENARIO_METHOD(AccountScenario::deposit_scenario);
    CPPTESK_ADD_SCENARIO_METHOD(AccountScenario::withdraw_scenario);
}
AccountScenario::~AccountScenario(void) {}

AccountScenarioEx::AccountScenarioEx(int limit, Account* impl) : AccountScenario(limit, impl)
{
    setup("Account Test"
          , UseVirtual::init, NoMethod::finish, &AccountScenarioEx::get_state_int
        );

    CPPTESK_ADD_SCENARIO_METHOD(AccountScenarioEx::random_withdraw_scenario);
}

bool AccountScenarioEx::get_state_scen()
{
    // std::cout << "AccountScenario::get_state_scen " << std::endl;
    return mediator.get_balance();
}

int AccountScenarioEx::get_state_int()
{
    // std::cout << "Current state: " << mediator.balance << std::endl;
    return mediator.balance;
}
