/*
 * Copyright 2010 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdio.h>

#include <hw/model.hpp>
#include <hw/message.hpp>

using namespace cpptesk::hw;

////////////////////////////////////////////////////////////////////////////////
// Message Model
////////////////////////////////////////////////////////////////////////////////
CPPTESK_MESSAGE(MSG)
{
public:
    MSG()
    {
        CPPTESK_ADD_FIELD(MSG::data);
    }
    
    virtual ~MSG() {}

    CPPTESK_SUPPORT_CLONE(MSG)

    CPPTESK_DECLARE_FIELD(data, 8);
};

////////////////////////////////////////////////////////////////////////////////
// Design Model
////////////////////////////////////////////////////////////////////////////////
CPPTESK_MODEL(DUT)
{
public:
////////////////////////////////////////////////////////////////////////////////
// Input interface
////////////////////////////////////////////////////////////////////////////////
    CPPTESK_DECLARE_IFACE(iface);

////////////////////////////////////////////////////////////////////////////////
// Design Stimuli
////////////////////////////////////////////////////////////////////////////////

    CPPTESK_DECLARE_PROCESS(process)
    {
        START();

        CPPTESK_DEBUG_PRINTF(DEBUG_USER, "[%03d]: thread: stage 0", process.pid);
        CYCLE();

        CPPTESK_DEBUG_PRINTF(DEBUG_USER, "[%03d]: thread: stage 1", process.pid);
        CYCLE();

        STOP();
    }

    CPPTESK_DECLARE_STIMULUS(stimulus)
    {
        int i;
        START_STIMULUS(SEQUENTIAL);

        // Create a new thread for processing the code
        start(&DUT::process, iface, message);

        // Execute the code sequentially within the given process
        start(&DUT::process, process, iface, message);

        for(i = 0; i < 9; i++)
        {
            CPPTESK_DEBUG_PRINTF(DEBUG_USER, "[%03d]: stage=%2d", process.pid, i);
            CYCLE();
        }

        CPPTESK_DEBUG_PRINTF(DEBUG_USER, "[%03d]: stage=%2d", process.pid, i);
        STOP();
    }

////////////////////////////////////////////////////////////////////////////////
// Design Mediators
////////////////////////////////////////////////////////////////////////////////
    virtual void setInputs()
    {
        CPPTESK_DEBUG_PRINT(DEBUG_INFO, "Call getInputs()");
    }

    virtual void getOutputs()
    {
        CPPTESK_DEBUG_PRINT(DEBUG_WARN, "Call getOutputs()");
    }

    virtual void simulate()
    {
        CPPTESK_DEBUG_PRINT(DEBUG_FAIL, "Call simulate()");
    }
};

////////////////////////////////////////////////////////////////////////////////
// Testbench
////////////////////////////////////////////////////////////////////////////////
int main()
{
    DUT dut;
    MSG msg;

    debug(DEBUG_INTERNAL);

    dut.initialize();

    for(int i = 0; i < 100; i++)
    {
        CPPTESK_CALL_PROCESS_OF(dut, DUT::stimulus, dut.iface, msg);
        dut.cycle();
    }

    CPPTESK_DEBUG_PRINT(DEBUG_USER, msg.compare(msg));

    CPPTESK_DEBUG_PRINT(DEBUG_INFO, "CPPTESKHW_VERSION=" <<  CPPTESKHW_VERSION);
    CPPTESK_DEBUG_PRINT(DEBUG_MORE, "This is the end");

    dut.finalize();

    return 0;
}

